Creature = class()

function Creature:__init(image)
  self.id = #CreatureList + 1
  CreatureList[self.id] = self
  self.name = ""
  
  self.gpath = image
  self.image = love.graphics.newImage(self.gpath)
  self.quadid = 1
  self.quads = {}
  self.drawmode = 1
  
  self.animtimer = 0
  self.walktimer = 0
  
  self.x = 0
  self.y = 0
  self.w = 16
  self.h = 19
  
  self.maxhp = 100
  self.hp = self.maxhp
  
  self.solid = true
end

function Creature:setName(name)
  self.name = name
end

function Creature:getX()
  return self.x
end

function Creature:getY()
  return self.y
end

function Creature:getId()
  return self.id
end

function Creature:onThink(dt)
  self.animtimer = self.animtimer + dt
  self.walktimer = self.walktimer + dt
  
end

function Creature:setQuad(id)
  self.quadid = id
end

function Creature:setMaxHealth(max)
  self.maxhp = max
end

function Creature:setSize(w, h)
  self.w = w
  self.h = h
end

function Creature:setWidth(w)
  self.w = w
end

function Creature:setHeight(h)
  self.h = h
end

function Creature:getWidth()
  return self.w
end

function Creature:getHeight()
  return self.h
end

function Creature:getMaxHealth()
  return self.maxhp
end

function Creature:addHealth(hp)
  if (self.hp + hp) > self.maxhp then
    self.hp = self.maxhp
  elseif (self.hp + hp) < 0 then
    self.hp = 0
  else
    self.hp = self.hp + hp
  end
end

function Creature:setPosition(x, y)
  self.x = x
  self.y = y
end

function Creature:getPosition()
  return {x = self.x, y = self.y}
end

function Creature:draw()
  if self.drawmode == 2 then
    love.graphics.draw(self.image, self.quads[self.quadid], self.x, self.y)
  else
    love.graphics.draw(self.image, self.x, self.y)
  end
end

function Creature:setDrawMode(mode)
  self.drawmode = mode
  if mode == 1 then
    self.quads = {}
  else
    self:setupQuads()
  end
end

function Creature:move(x, y)
  if not self:canWalk(self.x + x, self.y + y) then
    return
  end
  if x then
    self.x = self.x + x
  end
  
  if y then
    self.y = self.y + y
  end
end
function Creature:setupQuads()
  self.quadid = 1
  self.quads = {}
  local quadAmount = math.floor(self.image:getWidth() / self.w)
  
  for x=1, quadAmount do
    self.quads[x] = love.graphics.newQuad((x-1) * self.w, 0, self.w, self.h, self.image:getDimensions()) 
  end
end

function Creature:animate(from, to)
  if not (self.quadid < (to and to or #self.quads) and self.quadid >= (from and from or 1)) then
      self.quadid = (from and from or 1)
  end
  
  if self.animtimer > 0.2 then
    self.animtimer = 0
    if self.quadid < (to and to or #self.quads) and self.quadid >= (from and from or 1) then
      self.quadid = self.quadid + 1
    else
      self.quadid = (from and from or 1)
    end
  end
end

function Creature:getName()
  return self.name
end

function Creature:canWalk(toX, toY)
  local can = true
  
  if toX < 0 or toX >= CONFIG.WORLDW - (self:getWidth() * 1.62) or toY < 0 or toY >= CONFIG.WORLDH - (self:getHeight() * 1.48) then
    return false
  end
  for _, v in pairs(CreatureList) do
    if v:getId() ~= self:getId() and CheckCollision(toX,toY,self.w,self.h, v.x,v.y,v.w,v.h) then
      can = false
      break
    end
  end

  for _, v in pairs(MapObjects) do
    if v:isSolid() and CheckCollision(toX,toY,self.w,self.h, v.x,v.y,v.w,v.h) then
      can = false
      break
    end
  
  end

  return can
end