Layer = class()

function Layer:__init(image, width, height, displaywidth, displayheight)
  self.tilesize = 16
  self.width = width
  self.height = height
  self.displaywidth = math.floor(displaywidth / 16) + 1
  self.displayheight = math.floor(displayheight / 16) + 1
  
  self.imagepath = image
  self.image = love.graphics.newImage(image)
  self.image:setFilter("nearest", "linear")
  
  self.zoom = 1
  
  self.x = 0
  self.y = 0
  
  self.tilebatch = love.graphics.newSpriteBatch(self.image, self.displaywidth * self.displayheight)
  
  self.map = {}
  self.quads = {}
  self.defaultquad = nil
  
  self:setupQuads()
end

function Layer:getQuad(id)
  return self.quads[id]
end

function Layer:setMapQuad(x, y, id)
  if not self.map[y] then
    self.map[y] = {}
  end
  self.map[y][x] = self:getQuad(id)
end

function Layer:getMapQuad(x, y)
  local quad = nil
  if self.map[y] and self.map[y][x] then
    quad = self.map[y][x]
  end
  
  if quad == nil then
    quad = self.defaultquad
  end
  
  return quad
end

function Layer:setDefaultQuad(id)
  if i == -1 then
    self.defaultquad = nil
    return
  end
  self.defaultquad = self:getQuad(id)
end

function Layer:setupQuads()
  for y=0, math.floor(self.height / self.tilesize) do
    for x=0, math.floor(self.width / self.tilesize) do
      self.quads[#self.quads + 1] = love.graphics.newQuad(x * self.tilesize, y * self.tilesize, self.tilesize, self.tilesize,
    self.image:getWidth(), self.image:getHeight())
    end
  end
end

function Layer:update()
  self.tilebatch:bind()
  self.tilebatch:clear()
  
  for x=0, self.displaywidth-1 do
    for y=0, self.displayheight-1 do
        if self:getMapQuad(x , y) then
          self.tilebatch:add(self:getMapQuad(x, y), x * self.tilesize, y * self.tilesize)
        end
    end
  end
  
  self.tilebatch:unbind()
end

function Layer:syncPos(x, y)
  self.x = (-x)
  self.y = (-y)
  self:update()
end

function Layer:draw()
  love.graphics.draw(self.tilebatch, self.x + (love.graphics.getWidth() / 2), self.y + (love.graphics.getHeight() / 2) ,0, self.zoom, self.zoom)
end